<?php
/**
 * CakePHP(tm) : Rapid Development Framework (http://cakephp.org)
 * Copyright (c) Cake Software Foundation, Inc. (http://cakefoundation.org)
 *
 * Licensed under The MIT License
 * For full copyright and license information, please see the LICENSE.txt
 * Redistributions of files must retain the above copyright notice.
 *
 * @copyright Copyright (c) Cake Software Foundation, Inc. (http://cakefoundation.org)
 * @link      http://cakephp.org CakePHP(tm) Project
 * @since     0.2.9
 * @license   http://www.opensource.org/licenses/mit-license.php MIT License
 */
namespace App\Controller;

use Cake\Core\Configure;
use Cake\Network\Exception\NotFoundException;
use Cake\View\Exception\MissingTemplateException;
use App\Form\RequestForm;

/**
 * Static content controller
 *
 * This controller will render views from Template/Pages/
 *
 * @link http://book.cakephp.org/3.0/en/controllers/pages-controller.html
 */
class PagesController extends AppController
{

    public function initialize()
    {
        parent::initialize();
        $this->loadComponent('ComptoirServer');
    }

    public function isAuthorized($user)
    {

        return parent::isAuthorized($user); // TODO: Change the autogenerated stub
    }

    public function beforeFilter(\Cake\Event\Event $event)
    {
        $this->Auth->allow([
            'search',
        ]);
        parent::beforeFilter($event);

    }

    /**
     * Displays a view
     *
     * @return void|\Cake\Network\Response
     * @throws \Cake\Network\Exception\NotFoundException When the view file could not
     *   be found or \Cake\View\Exception\MissingTemplateException in debug mode.
     */
    public function display()
    {
        $path = func_get_args();

        $count = count($path);
        if (!$count) {
            return $this->redirect('/');
        }
        $page = $subpage = null;

        if (!empty($path[0])) {
            $page = $path[0];
        }
        if (!empty($path[1])) {
            $subpage = $path[1];
        }
        $this->set(compact('page', 'subpage'));

        try {
            $this->render(implode('/', $path));
        } catch (MissingTemplateException $e) {
            if (Configure::read('debug')) {
                throw $e;
            }
            throw new NotFoundException();
        }
    }

    public function search()
    {

            if (isset($this->request->data["order"])) {
                $sort = explode(".", $this->request->data["order"]);
                $this->request->data["sort"] = $sort[0];
                $this->request->data["direction"] = $sort[1];
            }
        $this->request->url = str_replace(["en/", "fr/"], "", $this->request->url);
            $datas = $this->ComptoirServer->get(COMPTOIR_SRV_API . $this->request->url ,'softwares', 'software',$this->request->data);
            $this->set('datas',$datas);
            $this->set('searchQuery',$this->request->data['search']);


        $reviewed = [
            "false" => __d("Forms","Search.Filter.NoFilterOnReview"),
            "true"=>__d("Forms","Search.Filter.WithReview"),
        ];

        $screenCaptured = [
            "false" => __d("Forms","Search.Filter.NoFilterOnScreenshot"),
            "true"=>__d("Forms","Search.Filter.WithScreenshot"),
        ];

        $used = [
            "false" => __d("Forms","Search.Filter.NoFilterOnUser"),
            "true"=>__d("Forms","Search.Filter.WithUser"),
        ];

        $hasServiceProvider = [
            "false" => __d("Forms","Search.Filter.NoFilterOnServiceProvider"),
            "true"=>__d("Forms","Search.Filter.ServiceProvider"),
        ];

        $order = [
            "softwarename.asc"=>__d("Forms","Search.Sort.softwarenameAsc"),
            "softwarename.desc"=>__d("Forms","Search.Sort.softwarenameDesc"),
            "created.asc" => __d("Forms","Search.Sort.createdAsc"),
            "created.desc" => __d("Forms","Search.Sort.createdDesc"),
            "modified.asc"=>__d("Forms","Search.Sort.modifiedAsc"),
            "modified.desc"=>__d("Forms","Search.Sort.modifiedDesc"),
            "average_review.asc"=>__d("Forms","Search.Sort.averageReviewsAsc"),
            "average_review.desc"=>__d("Forms","Search.Sort.averageReviewsDesc"),
        ];

        $this->set('reviewed',$reviewed);
        $this->set('used',$used);
        $this->set('hasServiceProvider',$hasServiceProvider);
        $this->set('order',$order);
        $this->set('screenCaptured',$screenCaptured);

        //For Social MEDIAS => OPENGRAPH
        $openGraph = [
            "title"  => \Cake\Core\Configure::read("OpenGraph.title"),
            "description" => \Cake\Core\Configure::read("OpenGraph.description"),
            "image" => \Cake\Core\Configure::read("OpenGraph.image"),
            "imageSize" => ['width' => '200', 'height' => '300']
        ];

        $card = [
            "title"  => \Cake\Core\Configure::read("OpenGraph.title"),
            "description" => \Cake\Core\Configure::read("OpenGraph.description"),
            "image" => \Cake\Core\Configure::read("OpenGraph.image"),
            "imageSize" => ['width' => '200', 'height' => '300']
        ];

        $this->set('openGraph', $openGraph);
        $this->set('card', $card);
    }
}
